Attribute VB_Name = "ClockProduct"
Option Explicit

Private Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)
Private Declare Function GetTickCount Lib "kernel32.dll" () As Long

Private pi As Double
Private firstDraw As Boolean

Private hourHand As Product
Private minuteHand As Product
Private secondHand As Product
Private msecHand As Product

Sub ClockMain()
    ' PI definition
    pi = 4 * Atn(1)
    
    Dim myProd As Product
    Set myProd = CATIA.ActiveDocument.Product
    
    ' get products
    Set secondHand = myProd.Products.Item(1)
    Set minuteHand = myProd.Products.Item(2)
    Set hourHand = myProd.Products.Item(3)
    Set msecHand = myProd.Products.Item(4)
    
    Dim startTime As Long
    startTime = GetTickCount
    
    Dim secondUpdate As Date
    secondUpdate = Now
    
    Dim msecBase As Long
    DrawFace myProd, 250

    ' main timer loop
    Do
        ' redraw every 20 millisecond
        If (GetTickCount - startTime) > 20 Then
            DrawWatch GetTickCount - msecBase
            startTime = GetTickCount
            
            ' every second reset milliseconds to properly update msec hand
            If DateDiff("s", secondUpdate, Now) Then
                secondUpdate = Now
                msecBase = GetTickCount
            End If
        End If
        
        ' keep application responsive
        DoEvents
        Sleep 5
    Loop While True
End Sub

Sub DrawFace(prod As Product, r As Double)
    Dim element As Product
    Set element = prod.Products.Item(5)
    
    Dim refElem As Product
    Set refElem = element.ReferenceProduct
    
    Dim i As Long
    Dim coords()
    
    For i = 1 To 12
        coords = Array(r * 0.9 * Cos(360 / 12 * i * pi / 180), r * 0.9 * Sin(360 / 12 * i * pi / 180))
        
        Dim copied
        Set copied = prod.Products.AddComponent(refElem)
    
        Dim pos(11)
        copied.Position.GetComponents pos
    
        pos(9) = coords(0)
        pos(10) = coords(1)
        pos(11) = -20
    
        copied.Position.SetComponents pos
        Rotate copied, 360 / 12 * i * pi / 180, False
    Next
End Sub

Function GetHourFraction(t As Date) As Double
    ' 12 hours boundary
    Const BOUNDARY As Double = 0.5
    
    Dim fraction As Double
    fraction = CDbl(t) - Int(t)
    
    If fraction <= BOUNDARY Then
        GetHourFraction = fraction / BOUNDARY
    Else
        GetHourFraction = (fraction - BOUNDARY) / BOUNDARY
    End If
End Function

' rotate clock hands
Sub Rotate(child, myAngle As Double, Optional moveToZero As Boolean = True)
    Dim pos(11)
    child.Position.GetComponents pos
    
    ' set x-axis rotation
    pos(0) = Cos(myAngle)
    pos(1) = Sin(myAngle)
    
    ' set y-axis rotation
    pos(3) = -Sin(myAngle)
    pos(4) = Cos(myAngle)
    
    If moveToZero Then
        ' origin point translation
        pos(9) = 0
        pos(10) = 0
    End If

    child.Position.SetComponents pos
End Sub

Sub DrawWatch(msec As Long)
    Dim currTime As Date
    currTime = Now
    
    Dim currSecond As Long
    currSecond = Second(currTime)
    
    Dim currMinute As Long
    currMinute = Minute(currTime)
    
    Dim currHour As Long
    currHour = Hour(currTime)
    
    Dim angleSecond As Double
    angleSecond = pi / 2 - 360 / 60 * currSecond * pi / 180
    
    Dim angleMinute As Double
    angleMinute = pi / 2 - 360 / 60 * currMinute * pi / 180
    
    Dim angleHour As Double
    angleHour = pi / 2 - 360 * GetHourFraction(currTime) * pi / 180
    
    Dim angleMillisecond As Double
    angleMillisecond = pi / 2 - 360 / 1000 * msec * pi / 180
    
    Rotate secondHand, angleSecond
    Rotate minuteHand, angleMinute
    Rotate hourHand, angleHour
    Rotate msecHand, angleMillisecond, False
End Sub
